VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsWindows"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'---------------------------------------------------------------------------------------
' Module    : clsWindows
' DateTime  : 30/04/2007 23:23
' Author    : Vb System Library
' Purpose   : Gerer les fenetres
' Copyright : (C) 2007 L'quipe Vb System Library
'              See "modPresentation" for more information
'---------------------------------------------------------------------------------------
' Librairie de fonctions permettant d'agir simplement et rapidement sur le systme.
' Copyright (C) 2007 - L'quipe Vb System Library
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the Free Software
' Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
'---------------------------------------------------------------------------------------


Option Explicit


' Constantes pour flasher une fentre
Public Enum FLASH_OPTIONS
    [stop flashing] = 0
    [flash caption] = &H1
    [flash taskbar button] = &H2
    [flash taskbar button and caption] = &H1 Or &H2
    [Flash continuously until the 'stop flashing' flag is set] = &H4
    [Flash continuously until the window comes to the foreground] = &HC
End Enum


'---------------------------------------------------------------------------------------
' Procedure : GetWindow
' DateTime  : 01/05/2007 00:11
' Author    : Vb System Library
' Purpose   : Renvoie un objet Thread
' Params    : hWnd : Handle de la fenetre
'             [Optionnel] RefreshInfos : Si = TRUE rafraichit les informations de la fenetre
'                                           = FALSE par dfaut
'---------------------------------------------------------------------------------------
Public Function GetWindow(ByVal hwnd As Long, Optional ByVal RefreshInfos As Boolean = False) As Window
    Set GetWindow = New Window
    GetWindow.GetWindowByHandle hwnd, RefreshInfos
End Function


'---------------------------------------------------------------------------------------
' Procedure : ListWindows
' DateTime  : 23/11/2006 14:37
' Author    : Vb System Library
' Purpose   : Renvoie la liste des fentres
' Params    : tabHWnd() est un tableau de Long (de 0  UBound) qui sera modifi  la sortie de la fonction.
'               Il contiendra les handles de toutes les fenetres ouvertes
'---------------------------------------------------------------------------------------
Public Sub ListWindows(ByRef tabHWnd() As Long)
    Dim CurrWnd     As Long
    Dim Compteur    As Long
    ' Redimensionne le tableau
    ReDim tabHWnd(0) As Long
    ' On part d'une premire fenetre : le bureau (la premire n'a pas d'importance)
    CurrWnd = GetWindowAPI(GetDesktopWindow(), GW_CHILD)
    Compteur = 0
    ' Tant qu'on a une fenetre
    While CurrWnd <> 0
        ReDim Preserve tabHWnd(Compteur) As Long
        tabHWnd(Compteur) = CurrWnd
        Compteur = Compteur + 1
        ' On prend la fenetre suivante
        CurrWnd = GetWindowAPI(CurrWnd, GW_HWNDNEXT)
    Wend
End Sub


'---------------------------------------------------------------------------------------
' Procedure : GetWindowCaption
' DateTime  : 03/12/2006 15:52
' Author    : Vb System Library
' Purpose   : Renvoie le titre de la fentre
' Params    : hWnd : handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function GetWindowCaption(ByVal hwnd As Long) As String
    Dim Length As Long
    ' On rcupre la longueur de la chaine et on formate une chaine  cette longueur
    Length = GetWindowTextLength(hwnd)
    GetWindowCaption = Space$(Length + 1)
    ' On rcupre le nom de la fenetre
    Length = GetWindowTextA(hwnd, GetWindowCaption, Length + 1)
    GetWindowCaption = Left$(GetWindowCaption, Len(GetWindowCaption) - 1)
End Function


'---------------------------------------------------------------------------------------
' Procedure : SetWindowCaption
' DateTime  : 24/12/2006 16:27
' Author    : Vb System Library
' Purpose   : Modifie le titre de la fenetre
' Params    :   hWnd : handle de la fenetre
'               Caption : Nouveau titre
'---------------------------------------------------------------------------------------
Public Function SetWindowCaption(ByVal hwnd As Long, ByVal Caption As String)
    SetWindowTextA hwnd, Caption
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetWindowPosition
' DateTime  : 23/12/2006 19:35
' Author    : Vb System Library
' Purpose   : Renvoie la position de la fenetre sous forme de type Rect
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function GetWindowPosition(ByVal hwnd As Long) As RECT
    Dim tmpRect As RECT
    GetWindowRect hwnd, tmpRect
    GetWindowPosition = tmpRect
End Function


'---------------------------------------------------------------------------------------
' Procedure : SetWindowPosition
' DateTime  : 23/12/2006 19:35
' Author    : Vb System Library
' Purpose   : Positionne une fenetre
' Params    : hWnd : Handle de la fenetre
'               Left, Top, Width, Height sont optionnels de type Long
'               Si un argument n'est pas prcis, alors sa valeur ne sera pas modifie
'               (ils sont dfinis ici en tant que Variant afin de pouvoir utiliser IsMissing)
'---------------------------------------------------------------------------------------
Public Function SetWindowPosition(ByVal hwnd As Long, Optional ByVal Left As Variant, Optional Top As Variant, Optional Width As Variant, Optional Height As Variant)
    Dim WndPl As WINDOWPLACEMENT
    WndPl.Length = Len(WndPl)
    GetWindowPlacement hwnd, WndPl
    If (Not IsMissing(Left)) And IsNumeric(Left) Then
        WndPl.rcNormalPosition.Right = WndPl.rcNormalPosition.Right - WndPl.rcNormalPosition.Left + Left
        WndPl.rcNormalPosition.Left = Left
    End If
    If (Not IsMissing(Top)) And IsNumeric(Top) Then
        WndPl.rcNormalPosition.Bottom = WndPl.rcNormalPosition.Bottom - WndPl.rcNormalPosition.Top + Top
        WndPl.rcNormalPosition.Top = Top
    End If
    If (Not IsMissing(Width)) And IsNumeric(Width) Then WndPl.rcNormalPosition.Right = WndPl.rcNormalPosition.Left + Width
    If (Not IsMissing(Height)) And IsNumeric(Height) Then WndPl.rcNormalPosition.Bottom = WndPl.rcNormalPosition.Top + Height
    SetWindowPlacement hwnd, WndPl
End Function


'---------------------------------------------------------------------------------------
' Procedure : EnableWindowOpacity
' DateTime  : 03/12/2006 16:01
' Author    : Vb System Library
' Purpose   : Active la transparence d'une fenetre (ncessite Win2k ou +)
'               Renvoie FALSE si la transparence ne peut pas etre applique
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function EnableWindowOpacity(ByVal hwnd As Long) As Boolean
    On Error GoTo Err
    SetWindowLongA hwnd, GWL_EXSTYLE, GetWindowLongA(hwnd, GWL_EXSTYLE) Or WS_EX_LAYERED
    EnableWindowOpacity = True
    Exit Function
Err:
    EnableWindowOpacity = False
End Function
' Change la valeur de la transparence (de 0 : transparent  255 : opaque)
Public Function ChangeWindowOpacity(ByVal hwnd As Long, ByVal Alpha As Byte) As Boolean
    On Error GoTo Err
    SetLayeredWindowAttributes hwnd, 0, Alpha, LWA_ALPHA
    ChangeWindowOpacity = True
    Exit Function
Err:
    ChangeWindowOpacity = False
End Function
' Dsactive la transparence
Public Function DisableWindowOpacity(ByVal hwnd As Long)
    On Error Resume Next
    SetWindowLongA hwnd, GWL_EXSTYLE, GetWindowLongA(hwnd, GWL_EXSTYLE) - WS_EX_LAYERED
End Function


'---------------------------------------------------------------------------------------
' Procedure : WindowVisible
' DateTime  : 31/12/2006 14:50
' Author    : Vb System Library
' Purpose   : Renvoie TRUE si la fenetre est visible, FALSE dans le cas contraire
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function WindowVisible(ByVal hwnd As Long) As Boolean
    WindowVisible = IsWindowVisible(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : ShowWindow
' DateTime  : 31/12/2006 14:54
' Author    : Vb System Library
' Purpose   : Permet de montrer une fenetre qui n'tait pas visible
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function ShowWindow(ByVal hwnd As Long)
    ShowWindowA hwnd, SW_SHOW
End Function


'---------------------------------------------------------------------------------------
' Procedure : HideWindow
' DateTime  : 31/12/2006 14:54
' Author    : Vb System Library
' Purpose   : Permet de cacher une fenetre qui tait visible
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function HideWindow(ByVal hwnd As Long)
    ShowWindowA hwnd, SW_HIDE
End Function


'---------------------------------------------------------------------------------------
' Procedure : RenameWindow
' DateTime  : 07/02/2007 21:35
' Author    : Vb System Library
' Purpose   : Change le texte d'une fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function RenameWindow(ByVal hwnd As Long, ByVal Caption As String)
    SetWindowTextA hwnd, Caption
End Function


'---------------------------------------------------------------------------------------
' Procedure : WindowEnabled
' DateTime  : 22/02/2007 00:49
' Author    : Vb System Library
' Purpose   : Renvoie TRUE si la fenetre est active
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function WindowEnabled(ByVal hwnd As Long) As Boolean
    WindowEnabled = IsWindowEnabled(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : EnableWindow
' DateTime  : 07/02/2007 21:35
' Author    : Vb System Library
' Purpose   : Active une fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function SetWindowEnabled(ByVal hwnd As Long)
    EnableWindow hwnd, True
End Function


'---------------------------------------------------------------------------------------
' Procedure : DisableWindow
' DateTime  : 07/02/2007 21:35
' Author    : Vb System Library
' Purpose   : Desactive une fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function SetWindowDisabled(ByVal hwnd As Long)
    EnableWindow hwnd, False
End Function


'---------------------------------------------------------------------------------------
' Procedure : CloseWindow
' DateTime  : 07/02/2007 21:35
' Author    : Vb System Library
' Purpose   : Ferme la fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function CloseWindow(hwnd As Long)
    SendMessage hwnd, WM_CLOSE, 0, 0
End Function


'---------------------------------------------------------------------------------------
' Procedure : EnableCloseButton
' DateTime  : 07/02/2007 21:37
' Author    : Vb System Library
' Purpose   : Permet d'activer la croix "Fermer" du menu systme d'une fentre lorsqu'elle tait grise
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function EnableCloseButton(ByVal hwnd As Long)
    ' on dgrise grce  true qui remet le menu  zro
    GetSystemMenu hwnd, True
    DrawMenuBar hwnd
End Function


'---------------------------------------------------------------------------------------
' Procedure : DisableCloseButton
' DateTime  : 07/02/2007 21:38
' Author    : Vb System Library
' Purpose   : Dsactive la croix fermer
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function DisableCloseButton(ByVal hwnd As Long)
    Dim hMenu As Long
    Dim nCount As Long
    hMenu = GetSystemMenu(hwnd, 0)
    nCount = GetMenuItemCount(hMenu)
    Call RemoveMenu(hMenu, nCount - 1, MF_REMOVE Or MF_BYPOSITION)
    Call RemoveMenu(hMenu, nCount - 2, MF_REMOVE Or MF_BYPOSITION)
    DrawMenuBar hwnd
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetWindowClass
' DateTime  : 25/02/2007 18:54
' Author    : Vb System Library
' Purpose   : Renvoie le nom de la classe de la fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function GetWindowClass(ByVal hwnd As Long) As String
    ' On prpare la chaine de caractre avec 255 espaces
    GetWindowClass = Space$(255)
    ' On rcure le nom de la classe
    GetClassNameA hwnd, GetWindowClass, 255
    ' On efface les caractres vides
    GetWindowClass = Left$(GetWindowClass, InStr(GetWindowClass, vbNullChar) - 1)
End Function


'---------------------------------------------------------------------------------------
' Procedure : IsTask
' DateTime  : 25/02/2007 18:52
' Author    : Vb System Library
' Purpose   : Renvoie TRUE si la fentre est une tche (apparait dans le Ctrl Alt Suppr)
'               Critres :      - Visible
'                               - Avoir un titre
'                               - Pas de parent
'                               - Pas de propritaire
'                               - a ne doit pas etre le TaskManager (Progman)
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function IsTask(ByVal hwnd As Long) As Boolean
    'le fentre doit tre visible
    If IsWindowVisible(hwnd) And (GetWindowLongA(hwnd, GWL_HWNDPARENT) = 0) And Not (GetWindowTextLength(hwnd) = 0) Then
        ' Vrifie que ce n'est pas le TaskManager
        If GetWindowClass(hwnd) <> "Progman" Then
            IsTask = True
        End If
    End If
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetWindowParentProcess
' DateTime  : 01/03/2007 18:48
' Author    : Vb System Library
' Purpose   : Permet de rcuprer l'ID du processus d'une fentre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function GetWindowParentProcess(ByVal hwnd As Long) As Long
    GetWindowThreadProcessId hwnd, GetWindowParentProcess
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetWindowParentThreadID
' DateTime  : 01/03/2007 18:48
' Author    : Vb System Library
' Purpose   : Permet de rcuprer l'ID du thread qui a cr une fenetre
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function GetWindowParentThreadID(ByVal hwnd As Long) As Long
    GetWindowParentThreadID = GetWindowThreadProcessId(hwnd, 0&)
End Function


'---------------------------------------------------------------------------------------
' Procedure : BringToFront
' DateTime  : 06/08/2007 17:01
' Author    : Vb System Library
' Purpose   : Permet de mettre une fentre au premier plan
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Sub BringWindowToFront(ByVal hwnd As Long)
    Call SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, _
        SWP_NOACTIVATE Or SWP_SHOWWINDOW Or SWP_NOMOVE Or SWP_NOSIZE)
End Sub


'---------------------------------------------------------------------------------------
' Procedure : DoNotBringToFront
' DateTime  : 06/08/2007 17:01
' Author    : Vb System Library
' Purpose   : Permet d'enlever une fentre du premier plan
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Sub DoNotBringWindowToFront(ByVal hwnd As Long)
    Call SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, _
        SWP_NOACTIVATE Or SWP_SHOWWINDOW Or SWP_NOMOVE Or SWP_NOSIZE)
End Sub


'---------------------------------------------------------------------------------------
' Procedure : BringToTop
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Purpose   : Permet d'afficher (pas permanent) une fentre du premier plan
' Params    : hWnd : Handle de la fenetre
'---------------------------------------------------------------------------------------
Public Function BringToTop(ByVal hwnd As Long) As Long
    BringToTop = BringWindowToTop(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetActiveWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Purpose   : Rcupre le handle de la fentre active
'---------------------------------------------------------------------------------------
Public Function GetActiveWindow() As Long
    GetActiveWindow = GetActiveWindowAPI
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetDesktopWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Purpose   : Rcupre le handle du desktop
'---------------------------------------------------------------------------------------
Public Function GetDesktopWindow() As Long
    GetDesktopWindow = GetDesktopWindowAPI
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetForegroundWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Purpose   : Rcupre le handle de la ForegroundWindow
'---------------------------------------------------------------------------------------
Public Function GetForegroundWindow() As Long
    GetForegroundWindow = GetForegroundWindowAPI
End Function


'---------------------------------------------------------------------------------------
' Procedure : FlashWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Purpose   : Permet de flasher une fentre
' Params    : hWnd : Handle de la fenetre
'             Options : options de la fonction
'             TimeOut : Intervale personnalise
'             NumberOfTime : nombre de flash  faire
'---------------------------------------------------------------------------------------
Public Function FlashWindow(ByVal hwnd As Long, Optional ByVal Options As FLASH_OPTIONS = [flash taskbar button and caption], _
    Optional ByVal TimeOut As Long = 0, Optional ByVal NumberOfTime As Long = 1) As Long
    
Dim FlashInfo As FLASHWINFO
    
    With FlashInfo
        .cbSize = Len(FlashInfo)
        .dwFlags = Options
        .dwTimeout = TimeOut
        .hwnd = hwnd
        .uCount = NumberOfTime
    End With
    
    FlashWindow = FlashWindowEx(FlashInfo)
End Function


'---------------------------------------------------------------------------------------
' Procedure : IsWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Params    : hWnd : Handle de la fenetre
' Purpose   : Dtermine si le handle dsigne une fentre
'---------------------------------------------------------------------------------------
Public Function IsWindow(ByVal hwnd As Long) As Boolean
    IsWindow = IsWindowAPI(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : LockWindowUpdate
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Params    : hWnd : Handle de la fenetre  locker. Une seule fentre possible  la fois.
'                    Si le paramtre est 0, la fentre locke sera dlocke.
' Purpose   : Locke ou dlocke une fentre
'---------------------------------------------------------------------------------------
Public Function LockWindowUpdate(ByVal hwnd As Long) As Long
    LockWindowUpdate = LockWindowUpdateAPI(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : SetActiveWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Params    : hWnd : Handle de la fenetre  mettre active
' Purpose   : Rend Active une fentre
'---------------------------------------------------------------------------------------
Public Function SetActiveWindow(ByVal hwnd As Long) As Long
    SetActiveWindow = SetActiveWindowAPI(hwnd)
End Function


'---------------------------------------------------------------------------------------
' Procedure : SetForegroundWindow
' DateTime  : 03/09/2007 13:38
' Author    : Vb System Library
' Params    : hWnd : Handle de la fenetre  rendre active et au premier plan
' Purpose   : Rend Active et au premier plan une fentre
'---------------------------------------------------------------------------------------
Public Function SetForegroundWindow(ByVal hwnd As Long) As Long
    SetForegroundWindow = SetForegroundWindowAPI(hwnd)
End Function
